/*

-----------------------------------

Malokal Remote

Vinzenz Müller
Leon Klaßen

Berlin 2015

-----------------------------------

*/

import geomerative.*;
import oscP5.*;
import netP5.*;
import controlP5.*;
import java.net.InetAddress;

// PUNKTE ------------------             
RPoint[][] rawGroupedPoints; // Speichert die berechneten Text-Punkte in Gruppen
//ArrayList<RPoint[][]> rawGroupedPoints = new ArrayList<RPoint[][]>();
// COMMANDS ----------------             
FloatList command_list = new FloatList();  // befehlsliste
int command_list_length=0;

// malokal ----------------             
int malokal_current_index=0;
boolean is_malokal_connected=false;
boolean is_malokal_available=false;
boolean is_malokal_available_first=true;
boolean is_malokal_running=false;
boolean is_malokal_pause=false;
int malokal_request_index=0;
int last_malokal_available=millis();

// OSC ----------------             
OscP5 malokalOscP5;
OscP5 touchOscP5;

// UI  -------------------
ControlP5 cp5;
int control_height=100;  //Control-Bar-Hoehe in Pixeln
int control_zero_height; // Virtueller Nullpunkt 
int paper_zero_height=0; // Virtueller Nullpunkt 
PFont control_font=createFont("Helvetica Neue", 12);
boolean please_drawPaper=false;
boolean is_mouse_active=false;
int last_maus_moved=millis();
boolean is_controls_enabled=true;

// Papier ----------------
int paper_width_mm=300;  // Größe des Papiers in mm   297*210   1900*10000
int paper_height_mm=300;
float vis_left_mm=0;    // abstand der visualisierung
float vis_top_mm=0;
float vis_width_mm; // größe der visualisierung
float vis_height_mm;

// virtuelles Papier ----------------
int virtual_paper_left=20;
int virtual_paper_top=20;
float virtual_paper_ppmm=1;    // Pixel je mm, für Visualisierung


//SETUP ---------------------------------------------------

void setup() {
  
  frame.setTitle("Malokal Remote");
  
  //CANVAS ------------------
  if (System.getProperty("user.name").equals("Leon")) {
    size(1100, 700, "processing.core.PGraphicsRetina2D");
  } else {
    size(1100, 700);
  }
  
 // frame.setResizable(true);
  
  control_zero_height=height-control_height;
  
  //frameRate(15);
  smooth();
  stroke(255);
  background(255);
  textFont(createFont("Helvetica Neue", 10));
  noStroke();

  //USER-INTERFACE -----------
  cp5 = new ControlP5(this);
  GenerateGeneralControl();
  cp5.setFont(control_font);
  cp5.setColorCaptionLabel(color(255));
  cp5.setColorValueLabel(color(255));
  Mode.activate(0);
  
  cp5.loadProperties((System.getProperty("user.name")+".properties"));
  
  //OSC ---------------------
  malokalOscP5 = new OscP5(this, malokal_listen_port);
  touchOscP5 = new OscP5(this, touch_listen_port);

 // Funktion des Setup Buttons ein mal aufrufen
// Setup();
 


}//Setup

void exit() {
  cp5.saveProperties((System.getProperty("user.name")+".properties"));
}



//DRAW ---------------------------------------------------

void draw() {
  
  // maus active ---------------------------
  if (is_mouse_active&&millis()-last_maus_moved>10000) { // nach 10 sekunden deaktivieren
     is_mouse_active=false;
  } 
  
  // dynamische framerate ---------------------------
  if (focused) {    /*mouseY>control_zero_height*/
    if (is_mouse_active) { // active ui
      frameRate(30);
     // fill(color(255,0,0));
      //rect(width-10,height-10,10,10);
    } else if (is_malokal_running) { // es passiert was
      frameRate(30);
    } else { // es passiert nichts
      frameRate(30);
    }
  } else { // es passtiert was, aber niemand schaut zu
    frameRate(0.5);
  }

  //Berechnung der Punkte und Visualisierung -------------------------------
  if (please_drawPaper) {
    generateCommands();
    drawPaper(); //im Tab DrawPaper
    please_drawPaper=false;
  }

  noStroke();

  //User_Interface--------------
  fill(60);
  rect(0, control_zero_height, width, control_height);
  
  //Control -------------
 // fill(50);
 // rect(73, control_zero_height, 389, control_height);
 
  fill(0);
  rect(73, control_zero_height+10, 1, control_height-20);
  rect(473, control_zero_height+10, 1, control_height-20);
  rect(652, control_zero_height+10, 1, control_height-20);  
  rect(813, control_zero_height+10, 1, control_height-20);  

  //Paper -------------
 // fill(50);
 // rect(472, control_zero_height, 203, control_height);
  

  //Availability--------------
  if (is_malokal_available) { 
    
    fill(okcolor);
    StopGo.unlock();
    Pause.unlock();
    
    if (!is_malokal_running) {
      enableControls();
      rect(0,0,width,2);
    } else {
      disableControls();
      fill(100,225,255);
      rect(0,0,width/(float)(command_list_length-1)*(float)malokal_request_index,2);
      fill(0,125,255);
      rect(0,0,width/(float)(command_list_length-1)*(float)malokal_current_index,2);
    }
  
  } else {  
    
    if (is_malokal_connected) {
      fill(tintcolor);
      rect(0,0,width,2);
    } else {
    
    }

    StopGo.lock();
    Pause.lock();
    
  }
  
  if (millis()-last_malokal_available>5000) { // länger als 5 sec kein signal
    is_malokal_available=false;
  } 
  
  //Pause--------------
  if (is_malokal_pause) {
    Pause.setCaptionLabel("Weiter");
  } else {
    Pause.setCaptionLabel("Anhalten");
  }
  
  //Running--------------
  if (is_malokal_running) {
    StopGo.setCaptionLabel("Abbrechen");
  } else {
    StopGo.setCaptionLabel("Zeichnen");
  }
  
} //draw

